#!/bin/sh
# $Id: t-tls-rqstcert-0.sh,v 1.4 2014/02/15 03:14:10 ca Exp $
#
# Copyright (c) 2014 Claus Assmann
#
# By using this file, you agree to the terms and conditions set
# forth in the license/LICENSE.3C file which can be found at the
# top level of this source code distribution.
#
# ----------------------------------------
# test TLS: (do not) request a cert; check verify values
# uses smtpc2 with TLS
# run smtpcnf.sh with different parameters
# ----------------------------------------
#

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

SRCPRG=${STE}/smtpc2
chkcto smtps MTA_USE_TLS
srchastls

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

SENDER=sender@local.dom
RCPT=user@local.host

cd ../qmgr || exit 1

rm -f ${ALIMAP} ${MTMAP} ${ACCMAP}
# create aliases
${CAT} > ${ALI} <<EOF
abuse:	local:
postmaster:	local:
user:	local:
sender: local:
EOF
# create map
${MM} < ${ALI}
${CAT} > ${ACC} <<EOF
cltaddr:${MTA_LOCALHOST}	relay
EOF
${MM} -t'	' -F ${ACCMAP} < ${ACC}

# ----------------------------------------
# test: send mails to local address, check verify=NOT
# i.e., no cert requested
TEST=1
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4;
}
smar { Log_Level = 12; nameserver = ${NS}; }
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4;
}
smtps {
Log_Level = 14;
daemon_address = localhost:${SRVPORT};
wait_for_server=4;
tls { flags = {dont_request_cert}};
}
EOF
ERR=false
if ${SHELL} ${SD}/smtpcnf.sh -Z smtpc2 -c "-@S" -r -E 1 -A 1 -X 1 -S${SENDER} -R${RCPT}
then
  if chkverify "NOT" ${S1}
  then
    :
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi

# ----------------------------------------
# for the next tests
${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4;
}
smar { Log_Level = 12; nameserver = ${NS}; }
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4;
}
smtps {
Log_Level = 14;
daemon_address = localhost:${SRVPORT};
wait_for_server=4;
}
EOF

# ----------------------------------------
# test: send mails to local address, check verify=OK
# i.e., cert requested and sent
TEST=2
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false
if ${SHELL} ${SD}/smtpcnf.sh -Z smtpc2 -c "-@S" -r -E 1 -A 1 -X 1 -S${SENDER} -R${RCPT}
then
  if chkverify "OK" ${S1}
  then
    :
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi

# ----------------------------------------
# test: send mails to local address, check verify=FAIL
# cert requested, but can't be verified
TEST=3
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false
if ${SHELL} ${SD}/smtpcnf.sh -Z smtpc2 -c "-@S -Xccert4.pem -Xkkey4.pem" -r -E 1 -A 1 -X 1 -S${SENDER} -R${RCPT}
then
  if chkverify "FAIL" ${S1}
  then
    :
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi

# ----------------------------------------
# test: send mails to local address, check verify=
# no cert (initialization does not fail for "empty" filenames)
TEST=4
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false
if ${SHELL} ${SD}/smtpcnf.sh -Z smtpc2 -c "-@S -Xc  -Xk " -r -E 1 -A 1 -X 1 -S${SENDER} -R${RCPT}
then
  if chkverify "NO" ${S1}
  then
    :
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi

# ----------------------------------------
# for the next tests
${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4; }
smar { Log_Level = 12; nameserver = ${NS}; }
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }
smtps {
  Log_Level = 14; wait_for_server=4;
  daemon_address = localhost:${SRVPORT};
  session_features certnotrqst { tls { flags = {dont_request_cert}} }
  flags = {access, lookup_session_conf};
}
EOF
${CAT} > ${ACC} <<EOF
smtps_session_conf:127.0.0.1	session_feature=certnotrqst;
EOF
rm -f ${ACCMAP}
${MM} -t'	' -F ${ACCMAP} < ${ACC}

# ----------------------------------------
TEST=5
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false
if ${SHELL} ${SD}/smtpcnf.sh -Z smtpc2 -c "-@S" -r -E 1 -A 1 -X 1 -S${SENDER} -R${RCPT}
then
  if chkverify "NOT" ${S1}
  then
    :
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi


# ----------------------------------------
cd ${THISDIR}
rm -f ${MTMAP} ${ALIMAP}

testend
