/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-conf-array-0.c,v 1.6 2011/01/04 16:28:47 ca Exp $")

#include "sm/io.h"
#include "sm/string.h"
#include "sm/net.h"
#include "sm/units.h"
#include "sm/sm-conf.h"
#define SM_CONF_BYTE_DEF 1
#define SM_CONF_KBYTE_DEF 1
#include "sm/sm-conf-byte.h"
#include "sm/sm-conf-prt.h"
#include "sm/bhtable.h"
#include "sm/test.h"


/*
 *  test program for array handling: with some kind of access key
 */

/* connection limits */
struct sc_cl_S {
	char		*sccl_name;
	char		*sccl_rcpt_domain;
	uint32_t	 sccl_max_conn;
	uint32_t	 sccl_max_rate;
	uint32_t	 sccl_max_ta;
};
typedef struct sc_cl_S sc_cl_T, *sc_cl_P;

/* array of connection limits */
struct sc_cls_S {
	sc_cl_T	*sccls_cl;
	uint	 sccls_n;
};
typedef struct sc_cls_S sc_cls_T, *sc_cls_P;

typedef struct sc_cnf_S		sc_cnf_T, *sc_cnf_P;
struct sc_cnf_S
{
	sm_magic_T	 sm_magic;
	uint32_t	 sc_cnf_max_conn;
	uint32_t	 sc_cnf_max_rate;
	uint32_t	 sc_cnf_max_ta;
	sc_cls_T	 sc_cnf_sc_cls;
};

sm_conf_definition_T sc_cl_defs[] = {
{ SM_CONF_DEF_MAGIC, "name",		sm_conf_type_section_title,
	offsetof(sc_cl_T, sccl_name),	0,
	NULL,	0 /*SM_CONF_FLAG_STRICTLY_REQUIRED*/,
	NULL, NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
{ SM_CONF_DEF_MAGIC, "rcpt_domain",		sm_conf_type_string,
	offsetof(sc_cl_T, sccl_rcpt_domain),	0,
	NULL,	0 /*SM_CONF_FLAG_STRICTLY_REQUIRED*/,
	NULL, NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "max_ta",		sm_conf_type_u32,
	offsetof(sc_cl_T, sccl_max_ta),	sizeof(uint32_t),
	NULL, SM_CONF_FLAG_DEFAULT_FROM_ENVIRONMENT,
	NULL, NULL,	NULL,
	"max ta"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "max_conn",		sm_conf_type_u32,
	offsetof(sc_cl_T, sccl_max_conn),	sizeof(uint32_t),
	NULL, SM_CONF_FLAG_DEFAULT_FROM_ENVIRONMENT,
	NULL, NULL,	NULL,
	"max conn"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "max_rate",		sm_conf_type_u32,
	offsetof(sc_cl_T, sccl_max_rate),	sizeof(uint32_t),
	NULL, SM_CONF_FLAG_DEFAULT_FROM_ENVIRONMENT,
	NULL, NULL,	NULL,
	"max rate"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
};

sm_conf_definition_T sc_cl_definitions[] = {
{ SM_CONF_DEF_MAGIC, "",	sm_conf_type_section,
	offsetof(sc_cnf_T, sc_cnf_sc_cls.sccls_cl), sizeof(sc_cl_T),
	NULL, SM_CONF_FLAG_DEFAULT_FROM_ENVIRONMENT,
	sc_cl_defs,
	NULL, NULL, "connection limits" },

{ SM_CONF_DEF_MAGIC, "n",	sm_conf_type_array_n,
	offsetof(sc_cnf_T, sc_cnf_sc_cls.sccls_n), sizeof(uint),
	NULL, 0,
	sc_cl_defs,
	NULL, NULL,
	"number of connection limits (implicit)" },

CONF_SENTINEL
};

sm_conf_definition_T sc_defs[] = {

{ SM_CONF_DEF_MAGIC, "max_ta",		sm_conf_type_u32,
	offsetof(sc_cnf_T, sc_cnf_max_ta),	sizeof(uint32_t),
	"99",	0,
	NULL, NULL,	NULL,
	"global max ta"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "max_conn",		sm_conf_type_u32,
	offsetof(sc_cnf_T, sc_cnf_max_conn),	sizeof(uint32_t),
	"10",	0,
	NULL, NULL,	NULL,
	"global max conn"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "max_rate",		sm_conf_type_u32,
	offsetof(sc_cnf_T, sc_cnf_max_rate),	sizeof(uint32_t),
	"100",	0,
	NULL, NULL,	NULL,
	"global max rate"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "connection_limits",		sm_conf_type_array,
	0,	/* offset */
	0,	/* size (# of elements) */
	NULL,	/* default */
	SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_DEFAULT_FROM_ENVIRONMENT|SM_CONF_FLAG_SECTION_DEFAULT_FROM_ANONYMOUS,
	sc_cl_definitions,
	NULL,  NULL,
	"connection limit declarations"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
};

static sc_cnf_T s;
static int Verbose = 0;
static bht_P cl_ht = NULL;

static int
crt_cl_ht(sc_cnf_P sc_cnf, int size)
{
	uint n, u;
	sm_ret_T ret;
	bht_entry_P bht_entry;

	cl_ht = bht_new(size, size * 2);
	SM_TEST_E(cl_ht != NULL);

	n = s.sc_cnf_sc_cls.sccls_n;
	for (u = 0; u < n; u++) {
		char *key;
		size_t keylen;

		key = sc_cnf->sc_cnf_sc_cls.sccls_cl[u].sccl_rcpt_domain;
		if (key == NULL)
			continue;
		keylen = strlen(key);
		ret = bht_add(cl_ht, key, keylen,
				(void *)&(sc_cnf->sc_cnf_sc_cls.sccls_cl[u]), &bht_entry);
	}

	return 0;

  error:
	if (cl_ht != NULL)
		bht_destroy(cl_ht, NULL, NULL);
	return -1;
}

static int
chk_cl_ht(const char *rcpt_domain)
{
	sc_cl_P	sccls_cl;

	sccls_cl = (sc_cl_P) bht_find(cl_ht, rcpt_domain, strlen(rcpt_domain));
	if (NULL == sccls_cl) {
		sm_io_fprintf(smioout, "rcpt_domain=%s, status=not_found\n", rcpt_domain);
		return -1;
	}
	sm_io_fprintf(smioout, "rcpt_domain=%s, max_ta=%u, max_conn=%u, max_rate=%u\n"
		, rcpt_domain, sccls_cl->sccl_max_ta, sccls_cl->sccl_max_conn
		, sccls_cl->sccl_max_rate);
	return 0;
}

/*
	bht_destroy(cl_ht, NULL, NULL);
*/


static int
process(char const *name, FILE *fp)
{
	uint u,n;
	int err;
	char const *e = NULL;
	sm_conf_T *stream;
	char buf[SM_CONF_ERROR_BUFFER_SIZE];

	if (((stream = sm_conf_new(name ? name : "*stdin*"))) == NULL) {
		sm_io_fprintf(smioerr, "error -- sm_conf_new() returns NULL!\n");
		return 1;
	}
	if ((err = sm_conf_read_FILE(stream, name, fp)) != 0) {
		sm_io_fprintf(smioerr, "%s: %s\n",
			name ? name : "*stdin*",
			sm_conf_strerror(err, buf, sizeof buf));
		while ((e = sm_conf_syntax_error(stream, e)) != NULL)
			sm_io_fprintf(smioerr, "%s\n", e);
		sm_conf_destroy(stream);
		return 2;
	}

	if ((err = sm_conf_scan(stream, sc_defs, 0, &s)) != 0) {
		sm_io_fprintf(smioerr, "(while scanning) %s: %s\n",
			name ? name : "*stdin*",
			sm_conf_strerror(err, buf, sizeof buf));
		while ((e = sm_conf_syntax_error(stream, e)) != NULL)
			sm_io_fprintf(smioerr, "%s\n", e);
		sm_conf_destroy(stream);
		return 3;
	}

	(void) sm_conf_prt_conf(sc_defs, &s, smioout);

	n = s.sc_cnf_sc_cls.sccls_n;
	err = crt_cl_ht(&s, n * 2);

	if (Verbose > 0) {
		sm_io_fprintf(smioerr, "n=%u\n", n);
		for (u = 0; u < n; u++) {
			sm_io_fprintf(smioerr, "rcpt_domain[%u]=%s\n", u,
				s.sc_cnf_sc_cls.sccls_cl[u].sccl_rcpt_domain);
		}
	}

	return err;
}

int
main(int argc, char **argv)
{
	int ai, c, ret;
	const char *fn;

	fn = NULL;
	ret = 0;
	while ((c = getopt(argc, argv, "f:V")) != -1) {
		switch (c) {
		  case 'f':
			fn = optarg;
			break;
		  case 'V':
			++Verbose;
			break;
		}
	}
	argc -= optind;
	argv += optind;

	sm_test_begin(argc, argv, "test array 0");
	if (fn != NULL) {
		ret = process(fn, NULL);
		SM_TEST_E(ret == 0);
		for (ai = 0; ai < argc; ai++) {
			(void) chk_cl_ht(argv[ai]);
		}
		goto end;
	}
	if (argc == 0)
		ret = process("*stdin*", stdin);
	else {
		for (ai = 0; ai < argc; ai++) {
			ret = process(argv[ai], NULL);
			if (ret != 0)
				break;
		}
	}
  end:
	SM_TEST(ret == 0);
  error:
	sm_io_flush(smioout);
	return sm_test_end();
}
