#!/bin/sh
# $Id: t-inh-0.sh,v 1.3 2012/11/05 15:24:57 ca Exp $
# Copyright (c) 2011 Claus Assmann
#
# By using this file, you agree to the terms and conditions set
# forth in the license/LICENSE.3C file which can be found at the
# top level of this source code distribution.
#

test -s stop && exit 1

# source directory of this (and other) shell script
SD=`dirname $0`

ERRS=0
CNF=inh.conf
OUT=inh.out
OK=inh.ok
PRG=./t-inh-0
GREP=grep

VERBOSE=false
if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

error()
{
    ERRS=`expr ${ERRS} + 1 `
	${VERBOSE} && echo "$0: test ${TEST} failed"
    test X"${MTA_STOPONERROR}" != X && exit 1
}

# --------------------------------------------
TEST=1
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
cat > ${CNF} <<EOF
flags = { starttls, auth }
session_features notls { flags = { -starttls}}
EOF
cat > ${OK} <<EOF
# session feature declarations
session_features notls {
  # flags to influence behavior per session
  flags={
    # offer AUTH
    auth,
  }
  # max ta
  max_ta=0;
}
# global flags to influence behavior
flags={
  # offer STARTTLS
  starttls,
  # offer AUTH
  auth,
}
# global max ta
max_ta=99;
tls {
  # bits
  bits=0;
}
EOF

if ${PRG} ${CNF} >${OUT} 2>&1
then
  if diff ${OK} ${OUT}
  then
    :
  else
    error
  fi
else
  error
fi

# --------------------------------------------
TEST=2
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
cat > ${CNF} <<EOF
flags = { starttls, auth }
session_features notls { flags = { -starttls, -auth}}
EOF
cat > ${OK} <<EOF
# session feature declarations
session_features notls {
  # max ta
  max_ta=0;
}
# global flags to influence behavior
flags={
  # offer STARTTLS
  starttls,
  # offer AUTH
  auth,
}
# global max ta
max_ta=99;
tls {
  # bits
  bits=0;
}
EOF

if ${PRG} ${CNF} >${OUT} 2>&1
then
  if diff ${OK} ${OUT}
  then
    :
  else
    error
  fi
else
  error
fi
fi
fi


# --------------------------------------------
# don't inherit access flag
TEST=3
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
cat > ${CNF} <<EOF
flags = { access, starttls, auth }
session_features mt { max_ta=1; }
EOF
cat > ${OK} <<EOF
# session feature declarations
session_features mt {
  # flags to influence behavior per session
  flags={
    # offer STARTTLS
    starttls,
    # offer AUTH
    auth,
  }
  # max ta
  max_ta=1;
}
# global flags to influence behavior
flags={
  # use access map
  access,
  # offer STARTTLS
  starttls,
  # offer AUTH
  auth,
}
# global max ta
max_ta=99;
tls {
  # bits
  bits=0;
}
EOF

if ${PRG} ${CNF} >${OUT} 2>&1
then
  if diff ${OK} ${OUT}
  then
    :
  else
    error
  fi
else
  error
fi
fi


# --------------------------------------------
# error: access flag not allowed in local options
TEST=4
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
cat > ${CNF} <<EOF
flags = { access, starttls, auth }
session_features mt {flags = { access, starttls, auth }}
EOF
cat > ${OK} <<EOF
(while scanning) inh.conf: invalid value
inh.conf: expected number, got 'access'
inh.conf:2: unexpected flag 'access'
EOF

if ${PRG} ${CNF} >${OUT} 2>&1
then
  error
else
  if diff ${OK} ${OUT}
  then
    :
  else
    error
  fi
fi
fi


# --------------------------------------------
# error: cacert_file not allowed in "local" tls section
TEST=5
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
cat > ${CNF} <<EOF
session_features tcacert {
  tls { cacert_file=myca.pem; } }
EOF
cat > ${OK} <<EOF
(while scanning) inh.conf: invalid value
inh.conf:2: option 'cacert_file' not allowed in this 'tls' subsection
EOF

if ${PRG} ${CNF} >${OUT} 2>&1
then
  error
else
  if diff ${OK} ${OUT}
  then
    :
  else
    error
  fi
fi
fi

# --------------------------------------------
# cert_file ok in "local" tls section
TEST=6
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
cat > ${CNF} <<EOF
session_features tcert {
  tls { cert_file=cert.pem; key_file=key.pem;
      } }
EOF
cat > ${OK} <<EOF
# session feature declarations
session_features tcert {
  # max ta
  max_ta=0;
  tls {
    # File with certificate in PEM format
    cert_file="cert.pem";
    # File with private key for certificate in PEM format
    key_file="key.pem";
  }
}
# global flags to influence behavior
flags={
  # offer STARTTLS
  starttls,
}
# global max ta
max_ta=99;
tls {
  # bits
  bits=0;
}
EOF

if ${PRG} ${CNF} >${OUT} 2>&1
then
  if diff ${OK} ${OUT}
  then
    :
  else
    error
  fi
else
  error
fi
fi



# ----------------
if test ${ERRS} -eq 0
then
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  exit 1
fi
