#!/bin/sh
# $Id: t-dkim-conf-0.sh,v 1.25 2013/12/13 02:31:41 ca Exp $
# Copyright (c) 2011 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# test dkim signing, using a configuration file
# ----------------------------------------

test -s stop && exit 1              

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../libdkim
fi
. ${SD}/common.sh

# ----------------------------------------
# sign based on From: address (domain match)
TEST=1
ERR=false
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
H="P7M5AsnZKX0CSpQl3w3u9XwzYQ7gDrSE5gSSQy/1W2E="
SIG="hU1eiDdhQCwLZh1jXoUNF9Sg++8hJfXb4823xLTxNK2y6nMc6M6/iRudoNbF8/ijn/by/PnVE94Jphmzbr9OCfwh0zy+L9wd8ajH7ge7zPjQgUvuiLIzf33GtHYSrSGG4ry+LkUJNcKvjxI7UDSRlnodX6X3V6dWpto6l06yNeU"

cat >$BODY <<EOF
From: user@example.com
To: recipient@sendmail.net
Subject: test DKIM

2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net
.
EOF

cat >$CONF <<EOF
sign {
   domain=example.net;
   selector=default;
   keyfile=${K}/private.key;
   from=example.com;
}

sign net {
   domain=example.net;
   selector=net;
   keyfile=${K}/net.key;
}
EOF

$PRG -t ${DKTIME} -f $CONF -m $BODY -h "bh=$H" -S "b=${SIG}" -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# sign with default selection
TEST=2
ERR=false
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then

H="P7M5AsnZKX0CSpQl3w3u9XwzYQ7gDrSE5gSSQy/1W2E="
SIG="hU1eiDdhQCwLZh1jXoUNF9Sg++8hJfXb4823xLTxNK2y6nMc6M6/iRudoNbF8/ijn/by/PnVE94Jphmzbr9OCfwh0zy+L9wd8ajH7ge7zPjQgUvuiLIzf33GtHYSrSGG4ry+LkUJNcKvjxI7UDSRlnodX6X3V6dWpto6l06yNeU"

cat >$BODY <<EOF
From: user@example.com
To: recipient@sendmail.net
Subject: test DKIM

2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net
.
EOF

cat >$CONF <<EOF

sign {
   domain=example.net;
   selector=default;
   keyfile=${K}/private.key;
}

sign net {
   domain=example.net;
   selector=net;
   keyfile=${K}/net.key;
   from=example.net;
}

EOF

$PRG -t ${DKTIME} -f $CONF -m $BODY -h "bh=$H" -S "b=${SIG}" -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# test with From: header spread over multiple lines.
TEST=3
ERR=false

H="P7M5AsnZKX0CSpQl3w3u9XwzYQ7gDrSE5gSSQy/1W2E="
SIG="efTYq8bJC10OWo0xvIsIJyrVjT+6gUfSSz1kAUUs4iu41TyANWSLhj9m9UHF0eHe1vhiPMnubdMxuEsynDd5OKFFfOom3fzCtkW8AmoslLuelIN4nuSNVnDOFTrwlmu6LMlrmEXYf+u/bghGGP+SAsCXcBqV9TIYvhgrXoVOwgo="

if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then

cat >$BODY <<EOF
From: "Joe A. User"
  <user@  
	example.net>
To: recipient@sendmail.net
Subject: test DKIM

2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net
.
EOF

cat >$CONF <<EOF
sign {
   domain=example.net;
   selector=default;
   keyfile=${K}/private.key;
}

sign net {
   domain=example.net;
   selector=net;
   keyfile=${K}/net.key;
   from=example.net;
}

EOF

$PRG -t ${DKTIME} -f $CONF -m $BODY -h "bh=$H" -S "b=${SIG}" -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi


# ----------------------------------------
# test with full address selection
TEST=4
ERR=false

H="P7M5AsnZKX0CSpQl3w3u9XwzYQ7gDrSE5gSSQy/1W2E="
SIG="db1H8CgR9dPBjfcL0x9VCPUkHxXzHgtYjAQmkfeK3JHE2UNSC58/Z1Pkn3TcVVqf3cufS60yKL2ZZx3Dl8hS1tqLiTEeHcsaiYB3pq5dsDhY6RhQ/IMo2UrI18yousjtUf14/excru8JzZfTXgmfJnPd7jGLxhGx7OFwxCis/tg="

if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then

cat >$BODY <<EOF
From: "Joe A. User" <user@example.net>
To: recipient@sendmail.net
Subject: test DKIM

2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net
.
EOF

cat >$CONF <<EOF
sign {
   domain=example.net;
   selector=default;
   keyfile=${K}/private.key;
}

sign net {
   domain=example.net;
   selector=net;
   keyfile=${K}/net.key;
   from=user@example.net;
}

EOF

$PRG -t ${DKTIME} -f $CONF -m $BODY -h "bh=$H" -S "b=${SIG}" -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi


# ----------------------------------------
# test with full address selection: no match
TEST=5
ERR=false

H="P7M5AsnZKX0CSpQl3w3u9XwzYQ7gDrSE5gSSQy/1W2E="
SIG="tpRycxqsfQuCp4xj62D/bLXWEd2VOgwUnB67sJcsRIK3FZJwWar9D4iyOrecSvhyvmmMQZoHZRZRpzHOTb1OnKL9Pg0FSBtUdiYuMjXDPYleGGlIqb4geBPJ1VjBGjVuKgcWmXc0ZVXoPbH/L0LV6LaNP+ZPevzSODHJqImj0y0="

if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then

cat >$BODY <<EOF
From: "Joe A. User" <user@example.net>
To: recipient@sendmail.net
Subject: test DKIM

2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net
.
EOF

cat >$CONF <<EOF
sign {
   domain=example.net;
   selector=default;
   keyfile=${K}/private.key;
}

sign net {
   domain=example.net;
   selector=net;
   keyfile=${K}/net.key;
   from=nomatch@example.net;
}

EOF

$PRG -t ${DKTIME} -f $CONF -m $BODY -h "bh=$H" -S "b=${SIG}" -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi


# ----------------------------------------
# test with partial address specification
TEST=6
ERR=false

H="P7M5AsnZKX0CSpQl3w3u9XwzYQ7gDrSE5gSSQy/1W2E="
SIG="db1H8CgR9dPBjfcL0x9VCPUkHxXzHgtYjAQmkfeK3JHE2UNSC58/Z1Pkn3TcVVqf3cufS60yKL2ZZx3Dl8hS1tqLiTEeHcsaiYB3pq5dsDhY6RhQ/IMo2UrI18yousjtUf14/excru8JzZfTXgmfJnPd7jGLxhGx7OFwxCis/tg="

if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then

cat >$BODY <<EOF
From: "Joe A. User" <user@example.net>
To: recipient@sendmail.net
Subject: test DKIM

2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net
.
EOF

cat >$CONF <<EOF
sign {
   domain=example.net;
   selector=default;
   keyfile=${K}/private.key;
}

sign net {
   domain=example.net;
   selector=net;
   keyfile=${K}/net.key;
   from=user@.net;
}

EOF

$PRG -t ${DKTIME} -f $CONF -m $BODY -h "bh=$H" -S "b=${SIG}" -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# only a default sign section
TEST=7
ERR=false

H="SnvK8tdz36/rX9Me8dgKSBnZn2A="
SIG="myny1258GvJJD9aOSxmr4v0EI7/h/rHCude1Vmq0DPBQA/xC8DRGrmKvIvfwS76OftzJHqj9sFbk9+GiySqzAJNll3W1KkfTwUzgNietexDwnlsUO1a3x/igPfCUKaCjxr5EAX3Y47uOjYu29GOa42xJxa5T72+lNOZTZ7NqxV4="

if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then

cat >$BODY <<EOF
From: user@example.com
To: recipient@sendmail.net
Subject: test DKIM

2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net
.
EOF

cat >$CONF <<EOF
signature_method=rsa-sha1;
sign {
   domain=example.net;
   selector=default;
   keyfile=${K}/private.key;
}
EOF

$PRG -t ${DKTIME} -f $CONF -m $BODY -h "bh=$H" -S "b=${SIG}" -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi


# ----------------------------------------
# bogus input: empty
TEST=8
ERR=false
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
cat >$BODY <<EOF
.
EOF
cat >$CONF <<EOF
sign { domain=example.net; selector=default; keyfile=${K}/private.key; }
EOF
$PRG -t ${DKTIME} -f $CONF -m $BODY -e 3 -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# bogus input: no header
TEST=9
ERR=false
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
cat >$BODY <<EOF

body
.
EOF
cat >$CONF <<EOF
sign { domain=example.net; selector=default; keyfile=${K}/private.key; }
EOF
$PRG -t ${DKTIME} -f $CONF -m $BODY -e 3 -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# bogus input: bogus header 
TEST=10
ERR=false
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
cat >$BODY <<EOF
bogus-header
.
EOF
cat >$CONF <<EOF
sign { domain=example.net; selector=default; keyfile=${K}/private.key; }
EOF
$PRG -t ${DKTIME} -f $CONF -m $BODY -e 3 -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# bogus input: header name way too long
# XREF: size of a header name! (256 in msg.c)
TEST=11
ERR=false
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
rm -f $BODY
i=0
while test $i -le 52
do
  printf "01234567890123456789012345678901234567890123456789012345678901234567890123456789"  >> $BODY
  i=`expr $i + 1 `
done
cat >>$BODY <<EOF
: hah

.
EOF
cat >$CONF <<EOF
sign { domain=example.net; selector=default; keyfile=${K}/private.key; }
EOF
$PRG -t ${DKTIME} -f $CONF -m $BODY -e 0x94040107 -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# duplicate header
TEST=12
ERR=false
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
H="P7M5AsnZKX0CSpQl3w3u9XwzYQ7gDrSE5gSSQy/1W2E="
SIG="ZJa7YmqhDWE1vYoXzg3skbeawy4XquuYavOSxTmKaCZ+aOLofr70cFHEKfkDCoxm8CKy3BuWavvbzYl5+nbIYrnkVnk2W3YAZ/L3dBHiYur1go1910s/1A9/cvyOzzJkbkA+25BaO0DJVbYkDiN3JYRRRDbhM+jZkSQ/d5Fm3WQ="

cat >$BODY <<EOF
From: user@example.com
To: other@example.net
To: recipient@sendmail.net
Subject: test DKIM

2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net
.
EOF

cat >$CONF <<EOF
sign {
   domain=example.net;
   selector=default;
   keyfile=${K}/private.key;
   from=example.com;
}

sign net {
   domain=example.net;
   selector=net;
   keyfile=${K}/net.key;
}
EOF

$PRG -t ${DKTIME} -f $CONF -m $BODY -h "bh=$H" -S "b=${SIG}" -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi


# ----------------------------------------
if test ${ERRS} -eq 0
then
  rm -f ${BODY} ${OUT} ${CONF}
  exit 0
else
  echo "${ERRS} error(s)"
  exit 1
fi
