/*
 * Copyright (c) 1990, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Chris Torek.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 *
 *	$Id: print.h,v 1.18 2014/04/19 15:55:30 ca Exp $
 */

#ifndef PRINT_H
#define PRINT_H 1

/* Flags used during conversion. */
#define ALT			0x0001		/* alternate form */
#define HEXPREFIX	0x0002		/* add 0x or 0X prefix */
#define LADJUST		0x0004		/* left adjustment */
#define LONGINT		0x0010		/* long integer */
#define LLONGINT	0x0020		/* long long integer */
#define SHORTINT	0x0040		/* short integer */
#define ZEROPAD		0x0080		/* zero (as opposed to blank) pad */
#define SANITIZE	0x0100		/* replace non-printable chars */
#define PTRINT		0x0200		/* (unsigned) ptrdiff_t */
#define SIZEINT		0x0400		/* (signed) size_t */
#define CHARINT		0x0800		/* 8 bit integer */
#define MAXINT		0x1000		/* largest integer size (intmax_t) */

/*
 * The size of the buffer we use as scratch space for many things
 * including integer conversions, hex arrays (these can be long),
 * module names, etc.
 */
#define BUF		256

/*
 *  INTERNAL for vfprintf and related functions!
 *  BEWARE, these `goto error' on error!
 */

int	sm_print(sm_file_T *, struct sm_uio *);

#define VFPRINT(ptr, len) do { \
	iovp->iov_base = (void *)(ptr); \
	iovp->iov_len = (len); \
	uio.uio_resid += (len); \
	iovp++; \
	if (++uio.uio_iovcnt >= NIOV) { \
		if (sm_print(fp, &uio)) \
			goto error; \
		iovp = iov; \
	} \
} while (0)

#if COND_PRINT
/* refers directly to printit! */
#define VFP_PRINT(ptr, len) do { if (printit) { VFPRINT(ptr, len); } } while (0)
#else
#define VFP_PRINT(ptr, len) VFPRINT(ptr, len)
#endif

#define VFP_FLUSH() do { \
	if (uio.uio_resid && sm_print(fp, &uio)) \
		goto error; \
	uio.uio_iovcnt = 0; \
	iovp = iov; \
} while (0)

/* Macros for converting digits to letters and vice versa */
#define to_digit(c)	((c) - '0')
#define is_digit(c)	((uint) to_digit(c) <= 9)
#define to_char(n)	((char) (n) + '0')

/*
 *  BEWARE, PAD uses `n'.
 */
#define PAD(howmany, with) do { \
	if ((n = (howmany)) > 0) { \
		while (n > PADSIZE) { \
			VFP_PRINT(with, PADSIZE); \
			n -= PADSIZE; \
		} \
		VFP_PRINT(with, n); \
	} \
} while (0)

	/*
	 *  To extend shorts properly, we need both signed and unsigned
	 *  argument extraction methods.
	 */
#define SARG() \
	(flags&LLONGINT ? va_arg(ap, longlong_T) : \
	    flags&LONGINT ? GETARG(long) : \
	    flags&PTRINT ? GETARG(ptrdiff_t) : \
	    flags&SIZEINT ? GETARG(ssize_t) : \
	    flags&SHORTINT ? (long) (short) GETARG(int) : \
	    flags&CHARINT ? (signed char)GETARG(int) : \
	    (long) GETARG(int))

/* ptrdiff_t should be cast to an unsigned type here, but which? ulong? */
#define UARG() \
	(flags&LLONGINT ? va_arg(ap, ulonglong_T) : \
	    flags&LONGINT ? GETARG(ulong) : \
	    flags&PTRINT ? GETARG(ptrdiff_t) : /* XXX */ \
	    flags&SIZEINT ? GETARG(size_t) : \
	    flags&SHORTINT ? (ulong) (ushort) GETARG(int) : \
	    flags&CHARINT ? (unsigned char)GETARG(int) : \
	    (ulong) GETARG(uint))

#define OKTOPRINT(ch, encode)  (ISPRINT(ch) && strchr((encode), (ch)) == NULL)
#define SANITIZE_SIZE(encode) do {           \
		if (flags & SANITIZE) {             \
			osize = size;                   \
			for (m = 0; m < osize; m++) {   \
				n2 = cp[m];                 \
				if (!OKTOPRINT(n2, (encode)))  \
					size += 2;              \
			}                               \
			if (prec >= 0 && size > prec)   \
				size = prec;                \
		}                                   \
	} while (0)

#if SIZEOF_INT == 4
# define SETLONG(flag)
#elif SIZEOF_LONG == 4
#define SETLONG(flag) flags |= LONGINT
#else
			ERROR: cannot find size for uint32
#endif

#define SM_PRT_IPV4(ipv4addr)							\
			SETLONG(flag);								\
			u32 = ipv4addr;								\
			src = (uchar *) &u32;						\
			wcp = buf + sizeof(buf);					\
			for (m = 3; m >= 0; m--) {					\
														\
				_uquad = src[m];						\
				while (_uquad >= 10) {					\
					*--wcp = to_char(_uquad % 10);		\
					_uquad /= 10;						\
				}										\
				*--wcp = to_char(_uquad);				\
				if (m > 0)								\
					*--wcp = '.';						\
			}											\
			SM_ASSERT(wcp >= buf);						\
			size = buf + sizeof(buf) - wcp;				\
			cp = wcp

/* the "funny" length check is supposed to avoid int overflow */
#define SM_PRT_HEX8(addr, len, zero)						\
			do {	\
				uint8_t *u8a, u8;	\
				xdigs = "0123456789ABCDEF";	\
				u8a = (uint8_t *)(addr);	\
				wcp = buf + sizeof(buf);		\
				if (sizeof(buf) > (len) && sizeof(buf) > (len) * 3 + 2) {	\
					for (m = (len) - 1; m >= 0; m--) {	\
						u8 = u8a[m];	\
						*--wcp = xdigs[u8 & 0x0f];	\
						u8 >>= 4;	\
						if (u8 != 0 || zero) *--wcp = xdigs[u8 & 0x0f];	\
						*--wcp = ':'; \
					}	\
					++wcp;	\
					size = buf + sizeof(buf) - wcp;		\
					cp = wcp;	\
				} else {	\
					(void) strlcpy(buf, "array_too_long", sizeof(buf));	\
					size = strlen(buf);	\
					cp = buf;	\
				}	\
			} while (0)

#define SM_PRT_HEX16(addr, len, zero)						\
			do {	\
				uint16_t *u16a, u16;	\
				xdigs = "0123456789ABCDEF";	\
				u16a = (uint16_t *)(addr);	\
				wcp = buf + sizeof(buf);			\
				if (sizeof(buf) > (len) && sizeof(buf) > (len) * 3 + 2) {	\
					for (m = (len) - 1; m >= 0; m--) {	\
						u16 = htons(u16a[m]);	\
						if (zero) {	\
							*--wcp = xdigs[u16 & 0x0f];	\
							u16 >>= 4;	\
							*--wcp = xdigs[u16 & 0x0f];	\
							u16 >>= 4;	\
							*--wcp = xdigs[u16 & 0x0f];	\
							u16 >>= 4;	\
							*--wcp = xdigs[u16 & 0x0f];	\
							u16 >>= 4;	\
						}	\
						else do {	\
							*--wcp = xdigs[u16 & 0x0f];	\
							u16 >>= 4;	\
						} while (u16);	\
						*--wcp = ':'; \
					}	\
					++wcp;	\
					size = buf + sizeof(buf) - wcp;		\
					cp = wcp;	\
				} else {	\
					(void) strlcpy(buf, "array_too_long", sizeof(buf));	\
					size = strlen(buf);	\
					cp = buf;	\
				}	\
			} while (0)

#if MTA_NETINET6
# define SM_PRT_IPV6(family, addr)						\
			if (AF_INET6 == (family)) {	\
				uint16_t *u16a, u16;	\
				xdigs = "0123456789abcdef";	\
				u16a = (uint16_t *)&(addr);	\
				wcp = buf + sizeof(buf);			\
				if (sizeof(buf) > INET6_ADDRSTRLEN) {	\
					for (m = 7; m >= 0; m--) {	\
						u16 = htons(u16a[m]);	\
						do {	\
							*--wcp = xdigs[u16 & 0x0f];	\
							u16 >>= 4;	\
						} while (u16);	\
						*--wcp = ':'; \
					}	\
					++wcp;	\
					size = buf + sizeof(buf) - wcp;		\
					cp = wcp;	\
				} else {	\
					(void) strlcpy(buf, "IPv6:error", sizeof(buf));	\
					size = strlen(buf);	\
					cp = buf;	\
				}	\
				break;	\
			}
#else
# define SM_PRT_IPV6(family, addr)
#endif

#define SM_PRT_UNIX(family, addr)		\
			if (AF_UNIX == (family)) {	\
				(void) strlcpy(buf, (addr)->sunix.sun_path, sizeof(buf));	\
				cp = buf;	\
				size = strlen(buf);	\
				break;	\
			}

#define SM_CHKFAM(family)	\
			if (AF_INET != (family)) {	\
				(void) strlcpy(buf, "IP:addr_family_unsupported", sizeof(buf));	\
				size = strlen(buf);	\
				cp = buf;	\
				break;	\
			}

#define UNSPEC_ADDR(addr) \
			if (NULL == (addr)) { \
				(void) strlcpy(buf, "unspec", sizeof(buf)); \
				size = strlen(buf); \
				cp = buf; \
				break; \
			}

#endif /* PRINT_H */
