/*
 *  producer program emulating smtps
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-prods-0.c,v 1.4 2010/09/10 03:21:38 ca Exp $")

#include "sm/assert.h"
#include "sm/error.h"
#include "sm/memops.h"
#include "sm/heap.h"
/* #include "sm/test.h" */
#include "sm/socket.h"
#include "sm/io.h"
#include "statethreads/st.h"
#include "sm/stsock.h"

#include "t-prdcns-0.h"

#define sm_sock_close(fd)	un_st_socket_close(fd)
#define SM_INVALID_SOCKET	NULL
#define SM_IS_INVALID_SOCKET(fd)	((fd) == NULL)

static ulong nsnd = 0;
static uint Max_wait_threads = T_QS;
static uint Max_cur_threads = T_QS;
static uint Wait_threads;
static uint Busy_threads;
static uint openrequests;
static int32_t ntid = 0;
static bool loggedrcvv = false;

/* useful macros */
#define WAIT_THREADS Wait_threads
#define BUSY_THREADS Busy_threads
#define TOTAL_THREADS (WAIT_THREADS + BUSY_THREADS)

uint Avail_entries = 1000;
#define AVAILENTRIES (Avail_entries - openrequests)

static st_netfd_t fd;

/* connect to server */
static int
ss_connect(void)
{
	int i;
	sm_ret_T ret;

	if (Verbose > 1)
		dbg_print("clt: connect\n");
	fd = SM_INVALID_SOCKET;
	ret = sm_err_perm(EINVAL);

	for (i = 1; i <= 5; i++) {
		ret = un_st_client_connect(MYSOCK, -1, &fd);
		if (sm_is_err(ret)) {
			dbg_print("clt: connect=%m\n", ret);
			if (sm_error_value(ret) == ECONNREFUSED) {
				st_sleep(i);
				continue;
			}
			break;
		}
		else
			break;
	}
	return ret;
}


/* send a value */
static int
p_sndv(int32_t v)
{
	int p;
	char buf[PBUFSZ];

	*((int *)buf) = v;
    p = st_write(fd, buf, PBUFSZ, 1000);
	if (p <= 0)
		return sm_err_perm(errno);
	++q_sent;
	return p;
}

/* receive a value */
static int
p_rcvv(int32_t *pv)
{
	int r;
	char buf[PBUFSZ];

	r = st_read(fd, buf, sizeof(buf), -1);
	if (r <= 0)
		return sm_err_perm(errno);
	++q_rcvd;
	*pv = *((int *)buf);
	return r;
}

/* send tid to "qmgr" and receive a value back: if <=0: it's -window-size */
static int
ss_hdl_session(int32_t tid)
{
	int r;
	int32_t v;

	if (Verbose > 2)
		dbg_print("session: before=sndv, tid=%d\n", tid);
	r = p_sndv(tid);
	if (sm_is_err(r)) {
		dbg_print("session: tid=%d, p_sndv=%m\n", tid, r);
		return r;
	}
	++nsnd;
	++openrequests;

	/*
	 *  could get any tid back... how much do we care here?
	 *  this program is just to test the throttling mechanism...
	 */

	do {
		if (Verbose > 2)
			dbg_print("session: before=rcvv, tid=%d\n", tid);
		r = p_rcvv(&v);
		if (sm_is_err(r)) {
			if (!loggedrcvv)
				dbg_print("session: tid=%d, p_rcvv=%m\n", tid, r);
			loggedrcvv = true;
			--openrequests;
			return r;
		}
		if (Verbose > 1)
			dbg_print("session: after=rcvv, tid=%d, v=%d\n", tid, v);
		if (v <= 0)
			Avail_entries = -v;
	} while (v <= 0);

	--openrequests;
	return 0;
}

/* handle "connections": start new thread if needed */
static void *
ss_hdl_connects(void *arg)
{
	int32_t tid;
	int r;

#define SS_NEW_THREADS(cmp)	\
	(WAIT_THREADS <= Max_wait_threads && TOTAL_THREADS cmp Max_cur_threads \
	 && AVAILENTRIES >= BUSY_THREADS && !stop)

	tid = ++ntid;
	while (SS_NEW_THREADS(<=)) {
		SM_ASSERT(WAIT_THREADS > 0);
		WAIT_THREADS--;
		BUSY_THREADS++;
		if (Verbose > 2)
			dbg_print("connects: where=1, tid=%d, Max_cur=%u, Avail=%u, Busy=%u, Wait=%u\n", tid, Max_cur_threads, Avail_entries, BUSY_THREADS, WAIT_THREADS);
		if (SS_NEW_THREADS(<)) {
			WAIT_THREADS++;
			if (st_thread_create(ss_hdl_connects, (void *)0, 0, 0) == NULL) {
				WAIT_THREADS--;
				dbg_print("connects: st_thread_create=%m\n", sm_err_temp(errno));
			}
			else if (Verbose > 2)
				dbg_print("connects: where=2, tid=%d, Max_cur=%u, Avail=%u, Busy=%u, Wait=%u\n", tid, Max_cur_threads, Avail_entries, BUSY_THREADS, WAIT_THREADS);
		}

		if (0 == Max_cur_threads || 0 == AVAILENTRIES || Verbose > 1)
			dbg_print("connects: where=3, tid=%d, Max_cur=%u, Avail=%u, Busy=%u, Wait=%u\n", tid, Max_cur_threads, Avail_entries, BUSY_THREADS, WAIT_THREADS);

		/*
		 *  XXX don't do that... keep this constant (value from qmgr)
		 *  and count "outstanding entries" instead to substract that
		 *  from this value.
		if (Avail_entries > 0)
			--Avail_entries;
		 */

		r = ss_hdl_session(tid);

		WAIT_THREADS++;
		SM_ASSERT(BUSY_THREADS > 0);
		BUSY_THREADS--;
		if (WAIT_THREADS > Max_cur_threads + 1)
			break;
		if (r != 0) {
			stop = true;
			break;
		}
		if (TOTAL_THREADS > 0)
			st_usleep(0); /* yield to other threads? */
	}
	SM_ASSERT(WAIT_THREADS > 0);
	WAIT_THREADS--;
	dbg_print("connects: where=terminate, tid=%d, Max_cur=%u, Avail=%u, Busy=%u, Wait=%u\n", tid, Max_cur_threads, Avail_entries, BUSY_THREADS, WAIT_THREADS);
	return NULL;
}

static int
ss_start_threads(void)
{
	int threads;
	uint u;

	/* Create connections handling threads */
	WAIT_THREADS = 0;
	BUSY_THREADS = 0;
	threads = 0;
	for (u = 0; u < Max_wait_threads; u++) {
		WAIT_THREADS++;
		if (st_thread_create(ss_hdl_connects, (void *) (long) 0, 0, 0) != NULL)
			threads++;
		else
			WAIT_THREADS--;
	}
	return threads;
}

static int
t_init(uint sz)
{
	int r;

	r = st_init();
	SM_TEST(r >= 0);
	if (r < 0)
		exit(r);
	r = ss_connect();
	if (r != 0)
		return r;
	if ((r = ss_start_threads()) <= 0)
		return r - 1;
	return 0;
}

static void
testev(uint sz)
{
	if (t_init(T_QS) != 0)
		return;
	if (SM_IS_INVALID_SOCKET(fd))
		return;

	while (nsnd < reps)
		st_sleep(1); /* pass control to other threads? */

	dbg_print("testev: nsnd=%lu, openrequests=%u\n", nsnd, openrequests);
}

static void
usage(const char *prg)
{
	sm_io_fprintf(smioerr, "%s: simple producer test\n", prg);
	sm_io_fprintf(smioerr, "usage %s [options]\n", prg);
	sm_io_fprintf(smioerr, "options\n");
	sm_io_fprintf(smioerr, "-M n  set Max_wait_threads to n\n");
	sm_io_fprintf(smioerr, "-m n  use n for size of iqdb\n");
	sm_io_fprintf(smioerr, "-r n  send n items in total\n");
	sm_io_fprintf(smioerr, "-s n  use n for size of window\n");
	sm_io_fprintf(smioerr, "-V    increase verbosity\n");
}

int
main(int argc, char *argv[])
{
	int c;
	uint sz;

	sz = 100;
	reps = 2 * sz;
	while ((c = getopt(argc, argv, "M:m:r:s:V")) != -1)
	{
		switch (c)
		{
		  case 'M':
			Max_wait_threads = atoi(optarg);
			break;
		  case 'm':
			Avail_entries = atoi(optarg);
			break;
		  case 'r':
			reps = atoi(optarg);
			break;
		  case 's':
			sz = atoi(optarg);
			Max_cur_threads = sz;
			break;
		  case 'V':
			Verbose++;
			break;
		  default:
			usage(argv[0]);
			return(1);
		}
	}
	testev(sz);
	return 0;
}
