#!/bin/sh
# $Id: t-smar-2.sh,v 1.25 2013/09/26 04:43:41 ca Exp $
# Copyright (c) 2003-2005 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ------------------------------------------------------------
# Description:
# Script to test MX/A expansion for RFC2821 addresses.
# Uses smar, t-smar-1, startsmar.sh, DNS
# Note: this program relies on the meta1.org DNS server.
# ------------------------------------------------------------
#

if test X"${MTA_NO_DNS_TEST}" != "X"
then
  echo "SKIPPED: $0"
  exit 0
fi

TPRG=./t-smar-1
SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../smar
fi
. ${SD}/common.sh

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

TEST=1
if echo "${SKIP} X" | grep -v " ${TEST} " > /dev/null
then

rm -f ${SOCK} ${PIDS}
# start SMAR
${SHELL} ${SD}/startsmar.sh ${SMAROPTS} ${SMAROPTS} || exit 1
rm -f ${OUT} ${IP} ${OK}

# ----------------------------------------
${VALGRIND} ${TPRG} -M -VVVV '<a@double10-1.meta1.org>' > ${OUT} 2>&1

grep '^IP' ${OUT} > ${IP}
cat > ${OK} <<EOF
IP[0]=10.0.0.1 [a000001]
EOF

if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
fi

# stop SMAR
for i in `cat ${PIDS}`
do
  kill ${i}
done
# ----------------
# thanks kids for not following the POSIX standard...
if test X"`uname`" = "XLinux"
then
  killall smar
fi
sleep 5
fi

# start SMAR

rm -f ${SOCK} ${PIDS}
# start SMAR
${SHELL} ${SD}/startsmar.sh -d9 ${SMAROPTS} || exit 1
rm -f ${OUT} ${IP} ${OK}

if echo "${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
TEST=2
# ----------------------------------------
${VALGRIND} ${TPRG} -M -VVVV '<a@double10-2.meta1.org>' > ${OUT} 2>&1

grep '^IP' ${OUT} > ${IP}
cat > ${OK} <<EOF
IP[0]=10.0.0.1 [a000001]
EOF

if diff ${OK} ${IP}
then
  :
else
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
fi
fi

if echo "${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
TEST=3
# ----------------------------------------
${VALGRIND} ${TPRG} -M -VVVV '<a@meta1-1-50.meta1.org>' > ${OUT} 2>&1

grep '^IP' ${OUT} > ${IP}
cat > ${OK} <<EOF
IP[0]=10.1.50.10 [a01320a]
IP[1]=10.1.50.11 [a01320b]
IP[2]=10.1.50.20 [a013214]
IP[3]=10.1.50.12 [a01320c]
IP[4]=10.1.50.21 [a013215]
EOF

if diff ${OK} ${IP} >/dev/null
then
  :
else

  # first two entries are random order, try again
cat > ${OK} <<EOF
IP[0]=10.1.50.11 [a01320b]
IP[1]=10.1.50.10 [a01320a]
IP[2]=10.1.50.20 [a013214]
IP[3]=10.1.50.12 [a01320c]
IP[4]=10.1.50.21 [a013215]
EOF

  if diff ${OK} ${IP}
  then
    :
  else
    ERRS=`expr ${ERRS} + 1 `
    echo "$0: test ${TEST} failed."
  fi
fi
fi

test ${ERRS} -eq 0 && rm -f ${OUT} ${IP} ${OK}

# stop SMAR
for i in `cat ${PIDS}`
do
  kill ${i}
done
# ----------------
# thanks kids for not following the POSIX standard...
if test X"`uname`" = "XLinux"
then
  killall smar
fi

exit ${ERRS}
